import unittest

from roseta import eat


class TestConvert(unittest.TestCase):
    def setUp(self) -> None:
        self.no_cls_data = {
            "零": (0, "no"),
            "零点八": (0.8, "no"),
            "八": (8, "no"),
            "八十": (80, "no"),
            "八十八": (88, "no"),
            "负八十八": (-88, "no"),
            "负零点八厘米": (-0.8, "cm"),
            "零点八厘米": (0.8, "cm"),
            "负零点八米": (-80, "cm"),
            "零点八米": (80, "cm"),
            "负一米八": (-180, "cm"),
            "一米八": (180, "cm"),
            "一米八七": (187, "cm"),
            "一米8": (180, "cm"),
            "一米87": (187, "cm"),
            "1米八": (180, "cm"),
            "1米八七": (187, "cm"),
            "1米8": (180, "cm"),
            "1米87": (187, "cm"),
            "负零点八斤": (-0.4, "kg"),
            "零点八斤": (0.4, "kg"),
            "零点八公斤": (0.8, "kg"),
            "零点八克": (0.0008, "kg"),
            "零点八千克": (0.8, "kg"),
            "八十八g": (0.088, "kg"),
            "八十八kg": (88, "kg"),
        }
        self.height_data = {
            # pattern1
            "负零点八": [(-0.8, "cm"), (-0.8, "m")],
            "负零点八厘米": [(-0.8, "cm"), (-0.008, "m")],
            "零点八米": [(80, "cm"), (0.8, "m")],
            "零点八厘米": [(0.8, "cm"), (0.008, "m")],
            "八米": [(800, "cm"), (8, "m")],
            "八厘米": [(8, "cm"), (0.08, "m")],
            "八十米": [(8000, "cm"), (80, "m")],
            "八十厘米": [(80, "cm"), (0.8, "m")],
            "八十七米": [(8700, "cm"), (87, "m")],
            "八十七厘米": [(87, "cm"), (0.87, "m")],
            # pattern2
            "-0.8米": [(-80, "cm"), (-0.8, "m")],
            "0.8米": [(80, "cm"), (0.8, "m")],
            "0.8厘米": [(0.8, "cm"), (0.008, "m")],
            "1米": [(100, "cm"), (1, "m")],
            "1厘米": [(1, "cm"), (0.01, "m")],
            "1.8米": [(180, "cm"), (1.8, "m")],
            "1.8厘米": [(1.8, "cm"), (0.018, "m")],
            "18米": [(1800, "cm"), (18, "m")],
            "18厘米": [(18, "cm"), (0.18, "m")],
            "180米": [(18000, "cm"), (180, "m")],
            "180厘米": [(180, "cm"), (1.8, "m")],
            # pattern3
            "负一米八": [(-180, "cm"), (-1.8, "m")],
            "一米八": [(180, "cm"), (1.8, "m")],
            "一米八七": [(187, "cm"), (1.87, "m")],
            "一米8": [(180, "cm"), (1.8, "m")],
            "一米87": [(187, "cm"), (1.87, "m")],
            "1米八": [(180, "cm"), (1.8, "m")],
            "1米八七": [(187, "cm"), (1.87, "m")],
            "1米8": [(180, "cm"), (1.8, "m")],
            "1米87": [(187, "cm"), (1.87, "m")],
            # pattern4
            "八十几厘米": [(85, "cm"), (0.85, "m")],
            "八十多厘米": [(82, "cm"), (0.82, "m")],
            "一米八几": [(185, "cm"), (1.85, "m")],
            "一米八多": [(182, "cm"), (1.82, "m")],
            # pattern5
            "80几": [(85, "cm"), (85, "m")],
            "80几米": [(8500, "cm"), (85, "m")],
            "80几厘米": [(85, "cm"), (0.85, "m")],
            "80多": [(82, "cm"), (82, "m")],
            "80多米": [(8200, "cm"), (82, "m")],
            "80多厘米": [(82, "cm"), (0.82, "m")],
        }
        self.weight_data = {
            # pattern1
            "负零点八": [(-0.8, "g"), (-0.8, "kg")],
            "负零点八斤": [(-400, "g"), (-0.4, "kg")],
            "零点八斤": [(400, "g"), (0.4, "kg")],
            "零点八公斤": [(800, "g"), (0.8, "kg")],
            "零点八克": [(0.8, "g"), (0.0008, "kg")],
            "零点八千克": [(800, "g"), (0.8, "kg")],
            "八十斤": [(40000, "g"), (40, "kg")],
            "八十公斤": [(80000, "g"), (80, "kg")],
            "八十克": [(80, "g"), (0.08, "kg")],
            "八十千克": [(80000, "g"), (80, "kg")],
            "八十八斤": [(44000, "g"), (44, "kg")],
            "八十八公斤": [(88000, "g"), (88, "kg")],
            "八十八克": [(88, "g"), (0.088, "kg")],
            "八十八千克": [(88000, "g"), (88, "kg")],
            "八十八g": [(88, "g"), (0.088, "kg")],
            "八十八kg": [(88000, "g"), (88, "kg")],
            # pattern2
            "-0.8": [(-0.8, "g"), (-0.8, "kg")],
            "-0.8斤": [(-400, "g"), (-0.4, "kg")],
            "0.8斤": [(400, "g"), (0.4, "kg")],
            "0.8公斤": [(800, "g"), (0.8, "kg")],
            "0.8克": [(0.8, "g"), (0.0008, "kg")],
            "0.8千克": [(800, "g"), (0.8, "kg")],
            "80斤": [(40000, "g"), (40, "kg")],
            "80公斤": [(80000, "g"), (80, "kg")],
            "80克": [(80, "g"), (0.08, "kg")],
            "80千克": [(80000, "g"), (80, "kg")],
            "88斤": [(44000, "g"), (44, "kg")],
            "88公斤": [(88000, "g"), (88, "kg")],
            "88克": [(88, "g"), (0.088, "kg")],
            "88千克": [(88000, "g"), (88, "kg")],
            "88g": [(88, "g"), (0.088, "kg")],
            "88kg": [(88000, "g"), (88, "kg")],
            "88G": [(88, "g"), (0.088, "kg")],
            "88KG": [(88000, "g"), (88, "kg")],
            "88Kg": [(88000, "g"), (88, "kg")],
            "88kG": [(88000, "g"), (88, "kg")],
            # # pattern3
            "负一斤八": [(-900, "g"), (-0.9, "kg")],
            "一斤八": [(900, "g"), (0.9, "kg")],
            "一斤8": [(900, "g"), (0.9, "kg")],
            "1斤八": [(900, "g"), (0.9, "kg")],
            "1斤8": [(900, "g"), (0.9, "kg")],
            # pattern4
            "八十几斤": [(42500, "g"), (42.5, "kg")],
            "八十多斤": [(41000, "g"), (41, "kg")],
            "八十几公斤": [(85000, "g"), (85, "kg")],
            "八十多公斤": [(82000, "g"), (82, "kg")],
            # pattern5
            "80几": [(85, "g"), (85, "kg")],
            "80多": [(82, "g"), (82, "kg")],
            "80几斤": [(42500, "g"), (42.5, "kg")],
            "80多斤": [(41000, "g"), (41, "kg")],
            "80几公斤": [(85000, "g"), (85, "kg")],
            "80多公斤": [(82000, "g"), (82, "kg")],
            "80几kg": [(85000, "g"), (85, "kg")],
            "80多kg": [(82000, "g"), (82, "kg")],
            "80几g": [(85, "g"), (0.085, "kg")],
            "80多g": [(82, "g"), (0.082, "kg")],
        }

    def test_eat(self) -> None:
        for key, value in self.no_cls_data.items():
            self.assertEqual(eat(key), value)

        for key, value in self.height_data.items():
            self.assertEqual(eat(key, cls="height", unit="cm"), value[0])
            self.assertEqual(eat(key, cls="height"), value[0])  # unit default: cm
            self.assertEqual(eat(key, unit="cm"), value[0])
            self.assertEqual(eat(key, cls="height", unit="m"), value[1])
            self.assertEqual(eat(key, unit="m"), value[1])

        for key, value in self.weight_data.items():
            self.assertEqual(eat(key, cls="weight", unit="g"), value[0])
            self.assertEqual(eat(key, unit="g"), value[0])
            self.assertEqual(eat(key, cls="weight", unit="kg"), value[1])
            self.assertEqual(eat(key, cls="weight"), value[1])  # unit default: kg
            self.assertEqual(eat(key, unit="kg"), value[1])

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()
